public abstract class PropertyChangedBase : INotifyPropertyChanged
{
	protected SynchronizationContext uiContext = SynchronizationContext.Current;

	public event PropertyChangedEventHandler PropertyChanged;

	#if NET45

	protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
	{
		if (uiContext != null)
		{
			uiContext.Send(delegate
				{
					if (PropertyChanged != null)
						PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
				}, null);
		}
		else
		{
			if (PropertyChanged != null)
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}
	}

	#else

	protected virtual void OnPropertyChanged(string propertyName)
	{
		if (uiContext != null)
		{
			uiContext.Send(delegate
				{
					if (PropertyChanged != null)
						PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
				}, null);
		}
		else
		{
			if (PropertyChanged != null)
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}
	}

	#endif

	#if !NET20 && !NET30

	protected virtual void OnPropertyChanged<T>(Expression<Func<T>> expression)
	{
		OnPropertyChanged(GetPropertyName(expression));
	}

	protected virtual void MultipleOnPropertyChanged(string[] propertys)
	{
		foreach (string property in propertys)
		{
			OnPropertyChanged(property);
		}
	}

	protected virtual void MultipleOnPropertyChanged<T>(Expression<Func<T>>[] expressions)
	{
		foreach (Expression<Func<T>> expression in expressions)
		{
			OnPropertyChanged(GetPropertyName(expression));
		}
	}

	protected string GetPropertyName<T>(Expression<Func<T>> expression)
	{
		MemberExpression memberExpression = expression.Body as MemberExpression;
		return memberExpression != null ? memberExpression.Member.Name : "";
	}

	#endif
}